/* timer.cpp
 * encapsulates the windows timer
 * Copyright (C) 2002 Michel Leunen
 * http://www.leunen.com/
 * michel@leunen.com
 *
 * This software is freeware and is provided at no charge to the user.
 * This software is provided 'as-is', without any express or implied warranty.
 * To the maximum extent permitted by applicable law will the author NOT be held
 * liable for any damages whatsoever arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose, including
 * commercial applications, and to freely redistribute it, with the following
 * restrictions:
 *
 * 1. If you use this software in a product, an acknowledgment in the product
 *    documentation would be appreciated.
 *
 * 2. If you use this software in a commercial application, please send me an email
 *	  detailing the application.
 *
 * 3. This notice may not be removed or altered from any source distribution.
 *
 * 4. It may be distributed by any means, provided that the original files
 * 		as supplied by the author remain intact and no charge is made other than
 * 		for reasonable distribution costs. If you are a vendor, if you want to
 * 		include this software in a compilation or in a magazine cover disk
 * 		you HAVE TO CONTACT ME to obtain my authorization before distribution.
 */

#include "timer.h"
#include <iostream>

//------------------------------------------------------------
HWND mlLib::Timer::hwnd_=0;
mlLib::Timer::Timer()
		:timerId_(0),Interval(1000)
{
	
}

mlLib::Timer::~Timer()
{
	if(hwnd_)
	{
		Stop();
		SendMessage(hwnd_,WM_CLOSE,0,0);
	}
}

bool mlLib::Timer::CreateHiddenWindow()
{
  WNDCLASS HiddenWindowClass ;
  HiddenWindowClass.style=0;
  HiddenWindowClass.lpfnWndProc=timerwndproc;
  HiddenWindowClass.cbClsExtra=0;
  HiddenWindowClass.cbWndExtra=0;
  HiddenWindowClass.hInstance=0;
  HiddenWindowClass.hIcon=0;
  HiddenWindowClass.hCursor=0;
  HiddenWindowClass.hbrBackground=0;
  HiddenWindowClass.lpszMenuName=NULL;
  HiddenWindowClass.lpszClassName="TimerHiddenWindow";

  if(!RegisterClass(&HiddenWindowClass))return 0;
  hwnd_ = CreateWindow(HiddenWindowClass.lpszClassName, "",
       WS_POPUP | WS_CAPTION | WS_CLIPSIBLINGS |
       WS_SYSMENU | WS_MINIMIZEBOX,
       GetSystemMetrics(SM_CXSCREEN)/2,
       GetSystemMetrics(SM_CYSCREEN)/2,
       0,0,0,0,NULL,NULL);
  if(hwnd_==NULL)return false;
  HMENU   SysMenu;
  SysMenu=GetSystemMenu(hwnd_,false);
  DeleteMenu(SysMenu,SC_MAXIMIZE,MF_BYCOMMAND);
  DeleteMenu(SysMenu,SC_SIZE,MF_BYCOMMAND);
  DeleteMenu(SysMenu,SC_MOVE,MF_BYCOMMAND);
  //SetWindowLong(hwnd_,GWL_USERDATA,(long)this);
  return true;
}

LRESULT CALLBACK mlLib::Timer::timerwndproc(HWND h,UINT msg,
				WPARAM wparam,LPARAM lparam)
{
	//Timer* ptr=reinterpret_cast<Timer*>(GetWindowLong(h,GWL_USERDATA));
	switch(msg)
	{
		case WM_CLOSE:
			DestroyWindow(h);
			break;
		case WM_DESTROY:
			PostQuitMessage(0);
			break;
		case WM_TIMER:
		{  
    	Timer* ptr=reinterpret_cast<Timer*>(wparam);
    	ptr->OnTimer();
    	break;
		}
		default:
			return DefWindowProc(h,msg,wparam,lparam);
	}
	return 0;
}

bool mlLib::Timer::Start()
{
	if(!hwnd_)
	{
		if(!CreateHiddenWindow())return false;
	}
	// In case Start() is called multiple times
	Stop();

	timerId_=reinterpret_cast<unsigned int>(this);
	if(!SetTimer(hwnd_,timerId_,Interval,NULL))return false;
	else return true;
}

bool mlLib::Timer::Stop()
{
	if(timerId_)
	{
		if(!KillTimer(hwnd_,timerId_))return false;
		timerId_=0;
	}
	return true;
}
